/**
 * SunSphereParameters defines global constants that configure SunSphere.<p>
 *
 * Copyright &copy; 1996 Martin Minow. All Rights Reserved.<p>
 * World map copyright &copy; 1992-97 Apple Computer Inc. All Rights Reserved.
 * Used by permission.<p>
 *
 * Permission to use, copy, modify, and redistribute this software and its
 * documentation for personal, non-commercial use is hereby granted provided that
 * this copyright notice and appropriate documentation appears in all copies. This
 * software may not be distributed for fee or as part of commercial, "shareware,"
 * and/or not-for-profit endevors including, but not limited to, CD-ROM collections,
 * online databases, and subscription services without specific license.<p>
 *
 * @author <a href="mailto:minow@pobox.com">Martin Minow</a>
 * @version 1.1
 * This applet is Java 1.1 compatible.
 */
package org.minow.applets.sunsphere;

public interface SunSphereParameters
{
    /*
     * To be a polite applet, we need a parameterInfo[][] matrix and, to
     * be a proper Java Bean, we need a number of strings for the BeanInfo
     * PropertyDescriptor[] vector. All of these strings are defined here
     * as public static values. As noted above, in the long term, these
     * should be moved into a separate class to facilitate localization.
     * pXXX is the property name, nXXX is the property display name, and
     * dXXX is the property's short description string.
     */
    public static final String pBackgroundColor	= "bgcolor";
    public static final String nBackgroundColor	= "Background Color";
    public static final String dBackgroundColor =
    	"The background color, #RRGGBB (in hex)";
    public static final String pTextColor	= "textcolor";
    public static final String nTextColor	= "Text Color";
    public static final String dTextColor =
    	"The text color, #RRGGBB (in hex)";
    public static final String pAutostart	= "autostart";
    public static final String nAutostart	= "Automatic Start";
    public static final String dAutostart	=
    	"True to start rotating when the applet loads";
    public static final String pSpinInterval	= "spininterval";
    public static final String nSpinInterval	= "Spin Interval";
    public static final String dSpinInterval	=
    	"The number of milliseconds between image rotations";
    public static final String pSpinning	= "spinning";
    public static final String nSpinning	= "Spinning";
    public static final String dSpinning	=
    	"Is the globe spinning.";
    public static final String pDate		= "date";
    public static final String nDate		= "Date";
    public static final String dDate		=
    	"Date and Time of the display";
    public static final String pLocation	= "location";
    public static final String nLocation	= "Location (Latitude, Longitude)";
    public static final String dLocation	=
    	"Observer's location (latitude and longitude)";	
    /* */
    public static final String pLongitude	= "longitude";
    public static final String nLongitude	= "Longitude";
    public static final String dLongitude	=
    	"The initial observer longitude";
    public static final String pLongitudeDelta	= "longitudedelta";
    public static final String nLongitudeDelta	= "Longitude Increment";
    public static final String dLongitudeDelta	=
    	"The number of degrees to rotate the earth when the East or West button is clicked.";
    public static final String pLongitudeSpin	= "longitudespin";
    public static final String nLongitudeSpin	= "Longitude Spin Degrees";
    public static final String dLongitudeSpin	=
    	"The number of degrees to rotate the earth East or West at each rotation interval";
    /* */
    public static final String pLatitude	= "latitude";
    public static final String nLatitude	= "latitude";
    public static final String dLatitude	=
    	"The initial observer latitude";
    public static final String pLatitudeDelta	= "latitudeincrement";
    public static final String nLatitudeDelta	= "Latitude Increment";
    public static final String dLatitudeDelta	=
    	"The number of degrees to rotate the earth when the North or South button is clicked.";
    public static final String pLatitudeSpin	= "latiudespin";
    public static final String nLatitudeSpin	= "Latiude Spin Degrees";
    public static final String dLatitudeSpin	=
    	"The number of degrees to rotate the earth North or South at each rotation interval";
    public static final String pLatitudeGrab	= "latitudegrab";
    public static final String nLatitudeGrab	= "Latitude Garb OK?";
    public static final String dLatitudeGrab	=
    	"Does moving the globe using the mouse also modify latitude"
    	+ " -- note that this is slow.";
    /* */
    /*
     * Rotate delta is used by Java Beans. This is a one-shot alternative
     * to setting the "Longitude Spin Degrees" parameter and hitting the button.
     */
    public static final String nRotate		= "by";
    public static final String dRotate		=
    	"Degree of rotation, use negative value for opposite direction.";
    /*
     * The parameterInfo[][] matrix is used by the browser.
     */
    public static final String parameterInfo[][] = {
        { pBackgroundColor,	"color",	dBackgroundColor	},
        { pSpinInterval,	"number",	dSpinInterval		},
        { pAutostart,		"boolean",	dAutostart		},
        /* */
        { pLongitude,		"number",	dLongitude		},
        { pLongitudeDelta,	"number",	dLongitudeDelta		},
        { pLongitudeSpin,	"number",	dLongitudeSpin		},
        /* */
	{ pLatitude,		"number",	dLatitude		},
	{ pLatitudeDelta,	"number",	dLatitudeDelta		},
	{ pLatitudeSpin,	"number",	dLatitudeSpin		},
	/* */
	{ pLatitudeGrab,	"boolean",	dLatitudeGrab		},
    };
        
    /*
     * defaultDelayInterval must be less than or equal to oneMinute.
     * We set it much too low to work around a bug in some implementations
     * where the Thread.interrupt() call doesn't awaken the runner thread.
     */
    public static final int defaultDelayInterval = 10000; /* Ten seconds */
    public static final int oneMinute		= 60000;

}



